/* get_enrate is a program that calls enrate for a computation
on a single complete file; more complicated calling programs could
step through the data with overlapping windows, but this calling program
is intended as an example to show how enrate is called.

enrate is a function that computes a
BW Hz (typ. 16) energy envelope for wideband speech in order to estimate
speaking rate as the time-varying center of mass for the low
frequency spectrum (skipping the first spectral point or two,
which correspond to the offset). The spectral estimation is all
done on a downsampled envelope, with sampling rate of LOWRATE 
(typ. 100-400 Hz).

This version is explicitly offline; that is, it computes the enrate
 for the complete chunk of data that it is fed.
*/
#include <stdio.h>
#include <math.h>
#include "enrate.h"
#include "functions.h"

#define DATBUFSIZE 1000000

main(int argc, char **argv)
{
	/* data arrays */
	float fdata[DATBUFSIZE];

	float enrate_val; /* 1st moment of wideband modulation spectrum */
	float enrate (float *, int, int );
	float ftmp;

	int samprate;

	int winlength; /* window length in sample pts */
	
	int nread;

	int ascii_in;
	int pad;
	int two_start;
	int start;

	int padlength, halfpadlength;

	float samp;
	short isamp;

	short *shortptr;

	int i, j;
	
	/* getopt variables */
	int c;
	extern int optind;
  	extern char *optarg;

	/* Defaults  (see enrate.h) */

	ascii_in = TRUE;
	samprate = DEF_SAMPRATE ;
	pad = FALSE;


	if(argc == 1)
	{
		printusage(argv[0]);
		exit(0);
	}

	/* Command line variable acquisition */
	while((c = getopt(argc, argv, "abS:P")) != -1)
	{
		switch(c)
		{
			case 'a':
				ascii_in = TRUE;
				break;

			case 'b':
				/* binary shorts in */
				ascii_in = FALSE;
				break;

			case 'S':
				samprate = atoi(optarg);
          			break;

			case 'P':
				/* Pad to 2 sec if under that */
				pad = TRUE;
				break;


			default:
 				printusage(argv[0]);
				break;
		}
	}

	shortptr = &isamp;

	/* default start point in energy spectrum */

	start = 2;

	/* Read in the raw data file */
	i = 0;
	while(1) /* infinite loop to read in data */
	{
		if(ascii_in == TRUE)
		{
			nread = scanf("%f", &ftmp);
			isamp = (short)ftmp;
		}
		else
		{
	 		nread = fread((char*)shortptr, sizeof(short), 1, stdin);
		}
		if(nread != 1) /* break when done with input */
		{
			break;
		}

		fdata[i++] = isamp;
		/* now loop back unless all data has been read */
	}
	winlength = i;

	if(pad == TRUE)
	{
		/* if under 1 second */
		if(winlength < ( samprate))
		{
			padlength = (samprate) - winlength;
			halfpadlength = padlength / 2;

			for(i=(winlength-1); i>=0; i--)
			{
				fdata[i+halfpadlength] = fdata[i];
			}
			for(i=0; i<halfpadlength; i++)
			{
				fdata[i] = 0.0;
			}
			for(i=winlength+halfpadlength; i<(samprate); i++)
			{
				fdata[i] = 0.0;
			}
			winlength = samprate;
		}
	}

	enrate_val = enrate(fdata, winlength, samprate);

	printf("%.2f\n", enrate_val);
	exit(0);
}


void printusage(char *fname)
{
  fprintf(stderr,"usage: %s [options]\n", fname );
  fprintf(stderr,"Where stdin and stdout are used,\n");
  fprintf(stderr,"output is always an ascii number,\n");
  fprintf(stderr,"at least one option is required, and\n");
  fprintf(stderr,"where options can be any of the following:\n");
  fprintf(stderr,"\t -a for input file ascii rather than shorts [TRUE]\n");
  fprintf(stderr,"\t -b for input file binary shorts, not ascii [FALSE]\n");
  fprintf(stderr,"\t\tNote that default input is ascii\n");
  fprintf(stderr,"\t -S Sampling frequency (in Hertz) [%d]\n",
          DEF_SAMPRATE );
  fprintf(stderr,"\t -P for padding with zeros to get 2 seconds min [FALSE]\n");
  exit( 0 );
}


